//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/** This class is the equivalent of BiomassTransfers and BiomassRates in the
 *  Stat package of SwarmGecko. Its purpose is just a data record for collecting
 *  biomass change statistics.
 */

package gecko.statglue;

import stats.*;
import gecko.*;
import java.awt.Color;

public class BiomassTransfers extends StatBuckets {
  // these are observations
  static public final int GROWTH           = 0;
  static public final int LOSS_STEAL       = 1;
  static public final int LOSS_STARVE      = 2;
  static public final int BIOMASS          = 3;

  // these are derived, not actually used in inBuckets
  static public final int NET_GROWTH       = 4;
  static public final int LOSSES           = 5;
  static public final int LAST_NET_GROWTH  = 6;
  static public final int NET_RATIO        = 7;
  static final private int numStats = 8;
  private boolean rateFlag;

  static private String[] names = { "Growth", "Theft", "Starve", "Biomass",
                             "NetGrowth", "Losses", "LastNetGrowth", "NetGrowthRatio" };
  static private Color[] colors = { Color.cyan, Color.red, Color.gray, Color.pink,
                             Color.green, Color.blue, Color.yellow, Color.magenta };

  /** One can request transfers as either amounts or rates (divided by total biomass).
   *  But each instance does only one. It matters only to the arithmetic done in
   *  step.
   */
  public BiomassTransfers(boolean rate) {
    super(numStats);
    rateFlag = rate;
  }
  /** Promotes inBucket to outBucket, then calculates the derived stats.
   *  The StatBroker -could- apply some of this arithmetic as weight or
   *  normalizer, but it would be a) confusing and b) a bookkeeping hassle
   *  when the statcycle changed. So here we're leaving the Transfer stats
   *  exactly as they should go on a plot.
   */
  public void step() {
    super.step();
    double statcycle = 1.0 * Globals.getStatCycle();
    double[] d = getBuckets();

    // notice that unlike species.biomass, this one =is= averaged over statcycle
    d[BIOMASS]      /= statcycle;
    d[GROWTH]      /= statcycle;
    d[LOSS_STEAL]  /= statcycle;
    d[LOSS_STARVE] /= statcycle;

    d[LOSSES]       = d[LOSS_STEAL] + d[LOSS_STARVE];
    d[NET_GROWTH]   = d[GROWTH]     - d[LOSSES];
    d[NET_RATIO]    = d[NET_GROWTH] - d[LAST_NET_GROWTH];

    // note this quantity is needed ABSOLUTE, not rate!
    overrideInBucket(LAST_NET_GROWTH, d[NET_GROWTH]);

    // rateFlag means want rates of change / biomass.
    if (rateFlag) {
      for (int i = 0; i < numStats; i++) {
        if ( (i == BIOMASS) || (i == NET_RATIO) ) {
          // don't divide either by biomass....
        } else {
          // since biomass already had statcycle divided, need to multiply
          // by statcycle here or end up with stat / (biomass*(statcycle)^2 )
          d[i] *= statcycle / d[BIOMASS];
        }
      }
    }
  }

  /** Let's put the ornate logic of what brokers to use to observe this
   *  silly thing here, as a public service.... Make 'em all--the caller's
   *  problem if not all wanted.
   */
  public BiomassBucketBroker[] makeBrokers(int resourceId) {
    BiomassBucketBroker[] brokers = new BiomassBucketBroker[numStats];
    for (int i=0; i < numStats; i++) {
      if ( (i == BIOMASS) || (i == NET_RATIO) ) {
        // never rates....
        brokers[i] = new BiomassBucketBroker(this, i, resourceId, false);
      } else {
        brokers[i] = new BiomassBucketBroker(this, i, resourceId, rateFlag);
      }
    }
    return brokers;
  }
  static public int getNumStats() {return numStats;}
  static public String[] getNames() { return names; }
  static public Color[] getColors() { return colors; }
}
