//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 2000, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator

/** MeanSigmaSkew statistics watch the mean, sigma (sqrt variance) and skew
 *  of whatever double-valued something distribution. Alas, the mean, sigma, and
 *  skew are not naturally sized to look nice together on the same plot. So...I
 *  want to show mean, mean+sigma, mean-sigma, sigma, and...just raw skew.
 *
 *  Written for observeRadius for Os for ScaredyWorld body size spread, vfb, 000209.
 */

package gecko.statglue;
import courseware.util.*;
import courseware.plot.*;
import courseware.jfcdep.*;
import stats.*;
import java.awt.Color;
import gecko.*;

public class MeanSigmaSkew extends StatSource implements IndexStatSource {
  // the stats we deliver
  static final public int MEAN_PLUS_SIGMA  = 0;
  static final public int MEAN             = 1;
  static final public int MEAN_MINUS_SIGMA = 2;
  static final public int SIGMA            = 3;
  static final public int SKEW             = 4;
  static final public int NUM_TRACES       = 5;
  static final private String[] labels =
    {  "Mean+Sigma", "Mean", "Mean-Sigma", "Sigma", "Skew" };

  protected double[] plottables;  // array of current values for the stats
  protected double sum;           // sum of raw values
  protected double sumsqr;        // sum of squares of raw values
  protected int count;            // number of values in the sums

  // can't use sum/sumsqr trick in calculating skew--must remember all vals.
  protected boolean doSkew;       // if not, don't remember raw values...
  protected double[] values;
  protected int allocChunk = 50; // rises if have to copy to bigger

  static private Color[] traceColors = null;

  public MeanSigmaSkew(boolean doSkew) {
    plottables = new double[NUM_TRACES];
    this.doSkew = doSkew;
    sum = sumsqr = 0;
    if (traceColors != null)
      return;
    Palette p = new Palette(NUM_TRACES-1, Palette.PASTEL);
    traceColors = new Color[NUM_TRACES];
    traceColors[MEAN]             = p.getColor(1);
    traceColors[MEAN_PLUS_SIGMA]  = p.getColor(2);
    traceColors[MEAN_MINUS_SIGMA] = p.getColor(2);
    traceColors[SIGMA]            = p.getColor(0);
    traceColors[SKEW]             = p.getColor(3);
  }

  public double getStat(int which) {return plottables[which];}

  public Color[] getColors() {return traceColors;}
  public String getTraceName(int which) {return labels[which];}

  public void tally(double rawValue) {
    count++;
    sum       += rawValue;
    double sqr = rawValue * rawValue;
    sumsqr    += sqr;

    if (doSkew) {
      if (values == null)
        values = new double[allocChunk];
      else if (count >= values.length) {
        allocChunk *= 2;
        double[] newarray = new double[allocChunk];
        System.arraycopy(values, 0, newarray, 0, count);
        values = newarray;
      }
      values[count - 1] = rawValue;
    }
  }

  /** Derive our stats from the tallied sum and sumsqr.
   */
  public void step() {
    if (count==0) {
      for (int i=0; i<NUM_TRACES; i++)
        plottables[i] = 0;
      return;
    }
    // mean?
    plottables[MEAN] = sum/count;
    double meansqr = plottables[MEAN] * plottables[MEAN];

    // sigma?
    double variance = (sumsqr/count) - meansqr;
    plottables[SIGMA] = Math.sqrt(variance);
    plottables[MEAN_PLUS_SIGMA] = plottables[MEAN] + plottables[SIGMA];
    plottables[MEAN_MINUS_SIGMA] = plottables[MEAN] - plottables[SIGMA];

    // skew?
    double skew = 0;
    for (int i=0; i<count; i++)
      skew += Math.pow(values[i] - plottables[MEAN], 3.0);
    skew /= count * Math.pow(plottables[SIGMA], 3.0);
    plottables[SKEW] = skew;

    // we have derived. So reset the count/sums
    count = 0;
    sum = sumsqr = 0.0;
  }

  public IndexStatBroker[] makeBrokers() {
    IndexStatBroker[] brokers = new IndexStatBroker[NUM_TRACES];
    for (int i=0; i < NUM_TRACES; i++) {
      brokers[i] = new IndexStatBroker(this, i);
    }
    return brokers;
  }
}
