//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990930, vfb, original Java
 * For Ofer's variant on Os' ScaredyGrazers, lifecycle stages by size.
 * For this to work, must be observing instanceof Critter. (Needs scalevolume
 * and scaleradius--let's not add another interface for that....)
 *
 * Uses min and max radius (anything lower or
 * higher goes in first or last), but the divisions between are either evenly
 * distributed by radius, or evenly distributed by biomass. Either way, the
 * divisions are labeled by radius (if biomass, the radius equivalent of that
 * biomass.)
 */
package gecko.statglue;
import gecko.*;
import gecko.zoo.*;
import courseware.jfcdep.TruncVal;

public class SizeStageObserver implements StageObserver, GetPropertyNamesable {
  // additional advertised Species parameters for this species
  private int numStages;  // number of sizes to divide agents into
  private boolean stageByRadius; // if false, increments on biomass
  private double stageMinRadius;
  private double stageMaxRadius;

  // criteria and labels
  private String[] stageLabels = null; // generated first time asked for
  private double[]  stageRadii = null; // generated first time asked for

  // rigmarole to get my parameters registered on menus
  private static String[] props = { "numStages", "stageByRadius",
    "stageMinRadius", "stageMaxRadius" };
  public String[] getPropertyNames() { return props;}

  private QuadCircleSpecies someCritter;  // cart/horse problem. need to know about some/any
                        // member of species to be observed to create the
                        // labels of the buckets.
  public SizeStageObserver(QuadCircleSpecies c) {
    someCritter = c;
  }
  // for convenience with composite species
  public SizeStageObserver(QuadCircleSpecies c, SizeStageObserver tocopy) {
    someCritter    = c;
    numStages      = tocopy.numStages;
    stageByRadius  = tocopy.stageByRadius;
    stageMinRadius = tocopy.stageMinRadius;
    stageMaxRadius = tocopy.stageMaxRadius;
  }

  // get/set public properties

  public void setNumStages(int v) {numStages = v; }

  public boolean isStageByRadius(){return stageByRadius;}

  public void setStageByRadius(boolean v) {stageByRadius = v;}

  public double getStageMinRadius(){return stageMinRadius;}

  public void setStageMinRadius(double v) {stageMinRadius = v;}

  public double getStageMaxRadius(){return stageMaxRadius;}

  public void setStageMaxRadius(double v) {stageMaxRadius = v;}


  /** Implement interface StageObservable.*/

  public int getNumStages() {
    if (stageRadii==null)
      makeStages();
    return numStages;
  }

  /** stage id of this critter (size0, ..., sizeStageN-1 */
  public int getStage(StageObservable agent) {
    if (!(agent instanceof QuadCircleSpecies))
      return 0;
    QuadCircleSpecies critter = (QuadCircleSpecies)agent;
    for (int i=0; i<numStages-1; i++) {
      if (critter.getRadius() < stageRadii[i])
        return i;
    }
    return numStages - 1;
  }

  /** stage plot label for a stage id (like "radius3.367") */
  public String getStageName(int i) {
    if (stageLabels == null) {
      makeStages();
    }
    if (stageLabels == null) {
      return "All";
    }
    return stageLabels[i];
  }

  /** First time I'm asked, need to build criteria and label list. */
  private void makeStages() {
    if (numStages == 0) {
      stageRadii = new double[] { Double.POSITIVE_INFINITY };
      stageLabels = new String[] { "All" };
    } else {
      double scaleradius = 1.0;
      double scalevolume = 1.0;
      if (someCritter instanceof Critter) {
        scaleradius = ((Critter)someCritter).getScaleRadius();
        scalevolume = ((Critter)someCritter).getScaleVolume();
      }

      stageRadii = new double[numStages];

      double radiusSpan = stageMaxRadius - stageMinRadius;
      double biomassSpan = Globals.radius2volume(stageMaxRadius, scaleradius, scalevolume)
                           - Globals.radius2volume(stageMinRadius, scaleradius, scalevolume);
      double radiusDelta = radiusSpan / numStages;
      double biomassDelta = biomassSpan / numStages;
      double radiusNext = stageMinRadius;
      double biomassNext;

      for (int i=0; i<numStages; i++) {
        if (stageByRadius)
          radiusNext += radiusDelta;
        else {
          biomassNext = Globals.radius2volume(radiusNext, scaleradius, scalevolume);
          biomassNext += biomassDelta;
          radiusNext = Globals.volume2radius(biomassNext, scaleradius, scalevolume);
        }
        stageRadii[i] = radiusNext;
      }

      stageLabels = new String[numStages];
      for (int j=0; j<numStages - 1; j++) {
        stageLabels[j] = "r <= " + TruncVal.truncVal(stageRadii[j], 6);
      }
      stageLabels[numStages - 1] = "r > " + TruncVal.truncVal(stageRadii[numStages - 2], 6);
    }
  }
}

