//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 991027, vfb, original Java
 * A LimitResourceQuad differs from a ResourceQuad in that it suffers no
 * depletion, and all feeders at this trough get the same value without any
 * competition. Basically, I'm using it to give the "quality" of the site, to
 * be interpreted as a multiplicative factor on the true driver of growth,
 * light. Hence, a LimitResourceQuad of value 1.0 means that Tree gets 100%
 * of the growth specified by the light competition. 1.1 would mean they do
 * better than light would permit--superenriched soil. 0.5 is poor soil.
 * Tree takes care of noticing if a tree crosses LimitResourceQuad's and weights
 * the limit factor by area crossing each limitquad.
 *
 * 001116, vfb, extension for resource gradient.
 * Two interpretations here: gradient (always left to right) over whole world
 * (all fragments) or gradient within this fragment. Hence, two access methods:
 * setQuality(left, right) and setQuality(left, right, worldMcr);
 */

package gecko.woods;

import gecko.*;
public class LimitResourceQuad extends Quad implements Siteable {
  private Site site;       // for collecting statistics
  private double qualityLeft;  // quality of limiting resources, left edge, usually 0<quality<1.0
  private double qualityRight;  // quality of limiting resources, right edge, usually 0<quality<1.0

  public LimitResourceQuad() {
    type = Quad.QUAD_FEED;
    setVisible(true);
  }
  public void setSite(Site s) {site = s;}
  public Site getSite() {return site;}
  public void setFrom(LimitResourceQuad other) {
    this.setMcr(other);

    site      = other.site;

    qualityLeft   = other.qualityLeft;

    qualityRight  = other.qualityRight;

  }

  /** note since gradient is presently only in x, the y is ignored. */

  public double getQuality(double x, double y) {

    double fractionIn = (x - this.lx) / (this.ux - this.lx);

    return qualityLeft + ((qualityRight - qualityLeft) * fractionIn);

  }

  /** setQuality(double) assumes left=right, no gradient */

  public void setQuality(double q){qualityLeft = qualityRight = q;}

  /** setQuality(double, double) is gradient over -this- mcr.*/

  public void setQuality(double left, double right){

    qualityLeft = left;

    qualityRight = right;

  }

  /** setQuality(double, double, Mcr) is gradient relative to -that- mcr.

   *  To work correctly, -my- mcr must be set -before calling this-.

   */

  public void setQuality(double left, double right, Mcr overMcr){

    // see getQuality

    double fractionIn = (this.lx - overMcr.lx) / (overMcr.ux - overMcr.lx);

    qualityLeft = left + ((right - left) * fractionIn);

    fractionIn = (this.ux - overMcr.lx) / (overMcr.ux - overMcr.lx);

    qualityRight = left + ((right - left) * fractionIn);

  }

  /** These for convenience of parameter file */

  public void setQualityLeft(double left) {qualityLeft = left;}

  public void setQualityRight(double right) {qualityRight = right;}


  /** This is how limitresourcequad's get "born" by Site. They don't die. */
  public Object newCopy() {
    getSite().notifyBirth();
    return this.copymom();
  }
  public LimitResourceQuad copymom() {
    LimitResourceQuad newbie = new LimitResourceQuad();
    newbie.setFrom(this);
    return newbie;
  }
  // not really relevant for now....but shouldn't be confused with "light"
  public int getResource() { return 2;}

  public void paint(QuadDisplay qd) {
    qd.drawRect(site.getColor(), lx, ly, ux, uy, false);

  }


  public String toString() {
    return "LimitResourceQuad" + Globals.pointerString(this) +
    "[qualityLeft=" + qualityLeft + ",qualityRight=" + qualityRight + ",super=" + super.toString() + "]";
  }
}




