//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  "mini-Sortie"
/**
 * 000412, vfb, modified height formula to use heightIntercept=1.35 ala
 *              Charlie Canham's latest paper.  Tested and found it made no
 *              difference, so removed it again.
 *
 * 991018, vfb, original Java
 * Base tree of the Trees model (miniSortie). Note that the landscape has
 * nothing to do with light competition in Woods. Each step, these agents do
 * their own calculation to find who's shading them. Also note these agents
 * don't do Gecko-style metabolism, having their own rules based on tree
 * field observations.
 *
 * Stats:
 * This species family redefines "Biomass" as "Basal Area", including renaming
 * the Globals.XXX_PLOT_NAMEs. If adding other species families, this could become
 * a problem. Note that biomassTransfers and biomassTheft are nulls--this species
 * family only does observeBiomass and observeBiomassGrowth.
 */

package gecko.woods;
import gecko.*;
import gecko.zoo.*;
import gecko.statglue.*;
import courseware.util.*;
import courseware.plot.*;
import courseware.jfcdep.*;
import java.awt.*;
import java.util.Random;

public class Tree extends QuadCircleSpecies implements StageObservable, Biomassable, ReapableTree {
  protected TreeParams params; // cut out of this object to save memory

  // The working settings for the species include x,y,z,radius,color in
  // QuadCircleSpecies. We're storing height in z. Note due South is
  // negative Y. All sun comes from due South. And, most importantly....
  // radius is my CROWN radius.
  protected double stemDiameter;      // my trunk diameter, in cm--relates to recruitRadius
  protected double crownBottom;       // z of crown bottom (z is crown top)
  protected double lastR;             // last size...
  protected double light;             // % of full sun at crown we managed to get (0..1)
  protected double nutrientFactor;    // 1.0 if quality of soil over crown area is 1.0
  protected double uncroppedRadius;   // reference radius to init %light on cropped crown
  protected double uncroppedCrownBottom; // crownbottom unnibbled by deer
  protected boolean fullGrownFlag;    // ReaperOld would like to know
  // kept around to avoid reallocations--always need one
  protected static McrCircle shadow = new McrCircle();
  protected static McrCircle shadow2 = new McrCircle();
  protected static McrCircle sky = new McrCircle();
  protected static McrCircle crossm = new McrCircle();
  protected static McrCircle equiv = new McrCircle();
  protected static McrCircle stem = new McrCircle();

  // we are going to want to keep track of which direction to grow in.
  // not an ability supplied by QuadCircleSpecies. Since light and cropping
  // are on diff cycle lengths, need to keep the wannago's separate.
  protected Direction cropWannago;  // scratchpad for where we're headed
  protected Direction lightWannago;  // scratchpad for where we're headed

  static {
    // since "Biomass" is unclear, rename Global plots
    Globals.BIO_PLOT_NAME        = "Basal Area";
    Globals.BIO_GROWTH_PLOT_NAME = "Growth vs. Basal Area";
    GrassResources resources = (GrassResources) Globals.getResources();
    resources.setNumResources(3);
    resources.setBioScale(0,1.0);  resources.setResourceName(0,"null");
    resources.setBioScale(1,1.0);  resources.setResourceName(1,"tree basal area");
    resources.setBioScale(2,1.0);  resources.setResourceName(2,"site quality");
  }

  /** Set all the derived sizes based on stemDiameter.
   *  CAREFUL: since radius is incremental, cannot call syncWithStem twice
   *  in one step, or the crown growth multiplies.
   */
  protected void syncWithStem() {
    // Pacala et al '93, eqn 1.
    double deltaD = stemDiameter - (2.0 * lastR);
    radius += params.crownToStem * deltaD;
    uncroppedRadius = params.crownToStem * stemDiameter;
    // There exists a stemDiameter which is the
    // minimum at which the tree is effectively maxHeight. I do not believe
    // the crown can get fatter if the tree can't get higher. The trunk may
    // keep getting fatter, but it isn't all alive, and can't support more crown.
    // Just replacing lost branches over time.
    // Made this decision when noticed that after 1000 years, there were about
    // 12 crowns per kilometer. I Don't Think So. So maxCrownRadius set to the
    // value equivalent to 98% of maxHeight. But that doesn't work either. The
    // answers for maple and yellow birch made sense, but hemlock were still
    // half the size of football fields. So had to add another parameter.
    // Also, if stunted in height, we're also stunted in crown width.
    double stuntedCrownRadius = nutrientFactor * params.maxCrownRadius;
    if (radius > stuntedCrownRadius) {
      radius = stuntedCrownRadius;
    }
    if (uncroppedRadius > stuntedCrownRadius) {
      uncroppedRadius = stuntedCrownRadius;
      fullGrownFlag = true;
    }
    // and NEVER permit radius to exceed uncroppedRadius
    if (radius>uncroppedRadius) radius=uncroppedRadius;

    // not a lot we can do about it--agent shoved offsite, has no area,
    // has no nutrientFactor--kill it.
    if (radius == 0) {
      nutrientFactor = 0;
      return;
    }

    // Pacala et al '93, eqn 1.
    // vfb, complicated 991207: deer damage crops crown from the bottom, so
    // restate so that deer damage isn't repeated beyond reason. If nothing
    // raises crownBottom (like deer does), is equivalent to old statement.
    double effCrownDepthToHeight = (z - crownBottom)/z;
    double stuntedMax = nutrientFactor * params.maxHeight;
    if (stuntedMax > params.maxHeight) stuntedMax = params.maxHeight;
    z = stuntedMax *
        ( 1 - Math.exp( - stemDiameter * (params.heightInitSlope/params.maxHeight) ) );
    if (z > Woods.getMaxHeight())
      Woods.setMaxHeight(z);
    if (z >= stuntedMax)
      fullGrownFlag = true;
    crownBottom = z * (1 - effCrownDepthToHeight);
    uncroppedCrownBottom = z * (1 - params.crownDepthToHeight);

// find maxCrown at 95% maxheight given Sortie's parameters
// also turned reproduction off so didn't get crowded.
/*
if ( (z > (.94 * params.maxHeight)) && (z < (.96 * params.maxHeight)) ) {
System.err.println("time " + Globals.getCurrentTime() + " : " +
getSpecies().getName() + " height = " + TruncVal.truncVal(z/params.maxHeight)
+ " of max, trunkD=" + stemDiameter + ", crownR = " + TruncVal.truncVal(radius));
}
*/

    setMcr();

  }
  public boolean stageObservableCheck() {return true;}

  /** Shall I die? Internally driven choice. "Die" is an order.
   *  Growth-dependent mortality and per-species random death. For the
   *  nonce, we die Now, we vanish Now. Later, want to make the hulks
   *  stick around a while?
   */
  public boolean shallIDie() {
    // species-specific random death? apply before full-sun escape clause
    // use Woods global to turn on/off so don't need to screw with all the
    // Species definitions to turn off.
    if ( Woods.isSpeciesRandomDeath() ) {
      if ( Globals.getRandomSource().nextDouble() < params.mortRandom ) {
        return true;
      }
    }

    // if my crown is the same size as my trunk, bail out, because that's
    // getting ridiculous--careful, cm vs. m
    if ( ( radius * 2.0 * 100 ) <= stemDiameter)
      return true;

    // if pushed offsite, have no nutrients left, could have height 0....
    if ( z == 0)
      return true;


    // check for growth-dependent mortality
    // according to p. 7 Pacala et al 93, this does not apply for
    // trees >20m high in full sun. Let's say full sun is > .95.
    if ((z >= 20) && (light >= 0.85))
//    if ((z >= 20) && (light >= 0.95)) // had in development in Nov'00
      return false;

    double stemR = 0.5 * stemDiameter;
// double growthRate = (stemR - lastR) / lastR;
double growthRate = stemR - lastR;
    // Pacala etal 93 eqn 6, result is chance over 2.5 years.
    double mortProb = params.mortMax * Math.exp( -1.0 * params.mortDecay * growthRate ) ;
    mortProb /= 2.5;
/*
if (radius < 0.5) {
System.err.println("---- time=" + Globals.getCurrentTime() + ",agent=" + this);
System.err.println("...mortProb=" + mortProb + ",uncroppedRadius=" + uncroppedRadius
+ ", light=" + light + ",growthRate=" + growthRate);
}
*/
    if ( Globals.getRandomSource().nextDouble() < mortProb ) {
      return true;
    }

    return false;
  }

  /** Logically, this belongs in the Reapers. However, since more than one
   *  Reaper needs it, I'm putting it here, so the Tree knows how to take
   *  damage, instead of the Reaper knowing the mechanics of inflicting it.
   *
   *  The crown is reduced in volume by the damageRate. Written for hurricanes.
   */
  public void crownDamage(double damageRate) {
    double crownH = z - crownBottom;
    double vol = radius * radius * Math.PI * crownH;
    vol *= 1.0 - damageRate;
    radius = Math.sqrt( vol / (Math.PI * crownH) );
  }
  /** Logically, this belongs in the Reapers. However, since more than one
   *  Reaper needs it, I'm putting it here, so the Tree knows how to take
   *  damage, instead of the Reaper knowing the mechanics of inflicting it.
   *
   *  The crown is reduced in volume by the damageRate, but only from the
   *  bottom, up to height.
   */
  public void crownDamageBottom(double damageRate, double height) {
    if (height < crownBottom) return;
    double depthAffected = height - crownBottom;
    // to take damageRate percent off volume, without changing radius,
    // you simply raise the bottom.
    double newDepth = depthAffected * (1.0-damageRate);
    crownBottom = height - newDepth;
  }

  /** An order from without--"You Die." aot "I die", at my choice. */
  public void die() {
    remove();  // from QuadTree / world
    Species s = getSpecies();
    s.notifyDeath();  // for population
    s.notifyDeath(this); // for stage
    s.notifyBioTrans(BiomassTransfers.LOSS_STARVE, getBiomass()); // for biomass
  }

  /** my shadow, with basic trig projecting it to given height.
   *  Returns null if no shadow at that height. Note that the
   *  answer is NOT a circle, but an oblong rectangle
   */
  private double shadowPercent;
  public McrCircle getShadow(double atHeight) {
    double dz = z - atHeight;
    shadowPercent = 1.0;
    if (dz <= 0)
      return null;
    // rem shadows cast straight N or +Y
    // because of this simplification, we can cast an oblong "circle"
    // so cast the crown first
    shadow.circleToSquare(x, y + dz * Woods.getSunSine(), radius);
    double shadowArea = shadow.getArea();
    // cast the crownbottom
    dz = crownBottom - atHeight;
    if (dz > 0) {
      shadow2.circleToSquare(x, y + dz * Woods.getSunSine(), radius);
      shadow.ly = shadow2.ly;
      shadowPercent = shadowArea / shadow.getArea();
    }
    if (shadowPercent > 1.0) {
System.err.println("still got percent > 1.0 = " + shadowPercent);
      shadowPercent = 1.0;
    }

    return shadow;
  }
  /** vfb, 000626 experiment: return casting area / shadow area for
   *  discounting darkness of shadow. Have to use quantity figured by
   *  getShadow, or get percentages >> 1.0.
   */
  public double getShadowPercent(Mcr shadow) {
    return shadowPercent;
  }

  /** don't use equiv-area for paint, since we're drawing ovals
   *  and is always at z = 0.
   */
  public McrCircle getPaintShadow() {
    // rem shadows cast straight N or +Y
    // because of this simplification, we can cast an oblong "circle"
    // so cast the crown first
    shadow.circleToInscribedSquare(x, y + z * Woods.getSunSine(), radius);
    // cast the crownbottom
    shadow2.circleToInscribedSquare(x, y + crownBottom * Woods.getSunSine(), radius);
    shadow.ly = shadow2.ly;
    return shadow;
  }

  /** Intersect the actual 3-d crowns and shove trunks. The shorter gets debited
   *  100% of the crown intersection, ie, it cannot grown into the crown above it.
   *  Also shoves trunk.
   *
   *  This is really a subroutine of bask.
   */
  protected void crownCrop(Lightbox lightbox, McrCircle equiv) {
    // set up for trunk aversion checks
    // my trunk-immune-to-seedlings area, applied in bask().
    double stemAversionR = Woods.thinningFactor * stemDiameter / 2.0; // that's cm
    stemAversionR /= 100; // that's m
    stem.circleToSquare(x, y, stemAversionR);

    // equiv and wannago preset by bask
    Quad plantRoot = Globals.getPlantRoot();
    Quad somequad = plantRoot.pickWin(equiv, plantRoot, false);
    Tree shadower;
    McBrick myBrick = new McBrick(this, crownBottom, z);
    McBrick crossBrick;
    double crownVolume = myBrick.getVolume();
    double crossVolume;
    while (somequad != null) {
      if ((somequad instanceof Tree) && (somequad != this)) {
        // first issue--crown aversion
        shadower = (Tree) somequad;
        if (Woods.isCrownCrop()) {
          // Does shadower affect me? already know our mcr's intersect, but not volume
          if (shadower.z > z) {
            if (shadower.crownBottom < z) {
              crossBrick = new McBrick(shadower, shadower.crownBottom, shadower.z);
              if ( myBrick.intersectMcb(crossBrick, crossBrick) != Mcr.MCR_DISJOINT ) {
                // the volume of the intersection
                double crossVol = crossBrick.getVolume();
                // crossVol would only debit at 100% if the crown it grew into
                // was a rock--maple and birch aren't *that* big an impediment.
                crossVol *= 1.0 - shadower.getParams().lightCoeff;
                // now find the volume of the column (inc. not intersecting)
                crossBrick.uz = z;
                crossBrick.lz = crownBottom;
                double overVol = crossBrick.getVolume();
                if (overVol == 0)
                  break;
                double debit = (overVol - crossVol) / overVol;
                lightbox.factorIn(crossBrick, debit );
                cropWannago.addDirection(x-shadower.x, y-shadower.y, debit);
              }
            }
          }
        }

        // second issue--trunk aversion
        // any babies (myage - theirage > 5 && theirage < 2) within
        // stem of me, I kill outright. Shadow is being reused here
        // as shadower's stem. Even if no chance of death, allow trunks to shove.
        double otherStemR = shadower.stemDiameter/(2.0 * 100.0);
        shadow2.circleToSquare(shadower.x, shadower.y, otherStemR);
        if ( shadow2.crossMcr(stem) ) {
          if ((shadower.getAge() < params.recruitAge)) {
            if (getAge() > params.recruitAge + 4) {
              shadower.die();
            } else { // both babies--I can move, but not kill
              cropWannago.addDirection(x - shadower.x, y - shadower.y, equiv.getArea());
            }
          // both big, both move.
          } else {
            cropWannago.addDirection(x - shadower.x, y - shadower.y, equiv.getArea());
          }
        }
      }
      somequad = somequad.pickWin(equiv, plantRoot, false);
    }
  }

  /** returns new value for skylight. The shadower blocks (maybe)
   *  some portion of this tree's sky. It's already been tested for
   *  being within candidate distance and height of this tree.
   */
  private double factorSky(double skylight, Tree shadower) {
    final double degrees45 = Math.PI * 45.0 / 180.0;
    final double skyConstants = 2 * Math.PI * (1.0 - Math.cos(degrees45));

    // For each candidate shadower tree, we construct a new sky dome.
    // Due to pleasant accident of 45 degrees with vertical, the center
    // of this sphere dome and the dome radius are easily found.
    double sky0z = z - radius; // sky dome origin z. It's x, y match this Tree.
    double skyR  = Globals.distance(x, y, sky0z, shadower.x, shadower.y, shadower.z);

    // From dome center, draw a line to closest top cylinder edge of shadower.
    // The angle formed between zenith => dome center => that point, = topAngle.
    // If topAngle is greater than 45 degrees, we can get out of here.
    double[] angle3pt = Globals.ptToward(shadower.x, shadower.y, shadower.z,
                                           x, y, shadower.z, shadower.radius);
    double dist2d = Globals.distance(x, y,
                               angle3pt[0], angle3pt[1]);
    // cheap check for > 45% angle & out -- avoid trig functions.
    if (dist2d > (shadower.z - sky0z))
      return skylight;

    double dist3d = Globals.distance(x, y, sky0z,
                               angle3pt[0], angle3pt[1], angle3pt[2]);
    // vfb, 000626, had atan....WRONG. pA33 PhysicsIII text.
    double topAngle = Math.asin(dist2d / dist3d); // angle from vertical
    topAngle = (Math.PI / 2.0) - topAngle; // phi, angle from horizontal

    // If passed the topAngle test, we need to go all the way here.
    // So might as well find lowAngle next. Almost always degrees45.
    double lowAngle = degrees45;
    if (sky0z < shadower.crownBottom) {
      // a negative distance "toward" means project away by distance.
      angle3pt = Globals.ptToward(shadower.x, shadower.y, shadower.crownBottom,
                         x, y, shadower.crownBottom, - shadower.radius);
      dist2d = Globals.distance(x, y,
                               angle3pt[0], angle3pt[1]);
      dist3d = Globals.distance(x, y, sky0z,
                               angle3pt[0], angle3pt[1], angle3pt[2]);
      // vfb, 000626, was....
      // lowAngle = Math.PI * 90.0 / 180.0; // pi/2
      // lowAngle -= Math.acos(dist2d / dist3d);
      // vfb, 000626, want...
      lowAngle = Math.asin(dist2d/dist3d); // angle from vertical
      lowAngle = (Math.PI / 2.0) - lowAngle; // angle from horizontal
    }

    // and widthAngle is angle from edge to edge of tree in 2d
    // vfb, 000626, OK, I buy this one.
    double widthAngle = 2.0 * Math.atan(shadower.radius / skyR);

    // HAVE THE INGREDIENTS! So do it.
    double skyR2 = skyR * skyR;
    double skyArea = skyConstants * skyR2;
    // vfb, 000626, think these cos were backward (cos(top) - cos(low))
    double blockArea = skyR2 * widthAngle * (Math.cos(lowAngle) - Math.cos(topAngle));
if (blockArea < 0.0) System.err.println("blockArea = " + blockArea + ", lowAngle=" + lowAngle + ", topAngle=" + topAngle);

    // vfb, 000626: this diffuses it in...
    skylight *= ( (skyArea - blockArea) + (blockArea * shadower.getParams().lightCoeff) )
                / skyArea;
    return skylight;
  }

  /** factors other tree's shadow onto sunbox */
  private void factorDirectSun(Lightbox sunbox, Tree shadower) {
    McrCircle shadow = shadower.getShadow(z);
    // now, factor in the fact that the shadow is bigger than the caster, thus
    // less dark.
    double lightCoeff = shadower.getLightCoeff();
    double darkCoeff = 1.0 - lightCoeff;
    darkCoeff *= shadower.getShadowPercent(shadow);
    lightCoeff = 1.0 - darkCoeff;

    if (shadow != null) {
      // shadow is NOT a circle anymore--don't use crossCircles
      // and note that though we picked candidates with sky mcr,
      // we find intersections with my crown (that's why getShadow(z)).
      double debit;
      if (equiv.intersectMcr(shadow, crossm) != Mcr.MCR_DISJOINT) {
        sunbox.factorIn(crossm, lightCoeff );
      }
    }
  }

 /** Bask = my year in the sun.
   * Set percent of full sun at crown based on neighbor shadowing.
   * Our quick&dirty equivalent to fish-eye photography.
   *
   * This routine is our "resource submodel" for light. If you want
   * another, subclass Tree and write your own. Might want to subclass
   * getShadow() as well.
   *
   * Although not related to light, the logic is very close, so here
   * we kill babies that have the misfortune to fall within 3x of our
   * stem radius. Which is probably overly generous--roots push hard.
   */
  public void bask() {
    // 000114: using cropYear=2 increased pop, so slowed simulation
    // using cropYear=5 caused huge overlaps, increased pop, slowed sim....
    final int cropYear  = 1;
    // lightYear=5 does speed things up a bit.
    final int lightYear = 5;

    double crownArea = Math.PI * radius * radius;
    double uncroppedArea = Math.PI * uncroppedRadius * uncroppedRadius;
    // this shouldn't happen, but when it does, it propagates NaN
    // throughout the Quadtree. Very confusing....
    if ((crownArea == 0) || (uncroppedArea == 0)) {
      return;
    }
    equiv.circleToSquare(this);
    Lightbox lightbox = new Lightbox(equiv, 1.0);

    // check for limiting resource and set nutrientFactor
    // moved before the if statement, 000811
    findNutrientFactor();

    if ((Globals.getCurrentTime() % cropYear) == 0) {
      cropWannago.zeroDirection();
      // Step 0a. Intersect the physical crowns. The shorter loses all
      // volume of intersection. As a side effect, since the pick aperture is
      // a subset of the crown aperture, we also shove trunks in crownCrop, so
      // as to avoid repeating the same traversal.
      crownCrop(lightbox, equiv);
      if (Woods.isCrownCrop()) {
        // lightbox is a scratch class that keeps clipping shadows and keeping
        // track of their light densities.... Crop crown with explicit boxes.
        Woods.setDiffusiveLightBox(false);

        // reset my radius and equivalent circle and lightbox based on crownCrop.
        double percentArea = lightbox.getLightDensity();
        if (percentArea == 0) {
          radius = 0;
          light = 0;
          return;
        }
        // don't permit roundoff to expand....
        if (percentArea > 1.0) percentArea = 1.0;
        double croppedArea = crownArea * percentArea;
        radius = Math.sqrt(croppedArea / Math.PI);
        equiv.circleToSquare(this);
        lightbox = new Lightbox(equiv, 1.0);
      }
    }

    // try skipping the light calculation except every 5th step.
    // doesn't work well from step, because do need the crown cropping
    // and trunk aversion more often.
    if ((Globals.getCurrentTime() % lightYear) != 0)
      return;
    lightWannago.zeroDirection();

    // let's try using diffusive light for shadows
    Woods.setDiffusiveLightBox(true);

    // Step 1: find the search area--which trees we'll question
    // how fast this program runs depends heavily on choosing as
    // small a search area as we possibly can. This is more-or-less
    // equivalent to pointing the fisheye camera.
    double skyHigh = Woods.getMaxHeight();
    double skydz = skyHigh - z;
    if (skydz < 0) skydz = 0;
    // the sky circle's center is mine, to start at least
    // the question is its radius, projected up to catch all sun from
    // within ( 45 degrees at this writing ) of vertical.
    // *** as of now, we're going to require 45 degrees--permits some tricks.
    // The sky mcr we're building here is the one used to pick candidate
    // trees for further consideration. It is NOT used to calculate light.
    double SIN45 = Math.sin(Woods.maxSunAngle);
    double skyR = radius + skydz * SIN45;
    // turn that into an equivalent-area square--rough at this expansion!
    sky.circleToSquare(x, y, skyR);

    // can crop the sky picker in this case
    if (Woods.weightDirectSun == 1.0) {
      sky.uy = y + skyR - (skydz * Woods.getSunSine() );
    }

    double skylight = 1.0; // light from sky before cutting.
    Lightbox sunbox = lightbox; // for direct sun (minus shadows)

    // Step 2: Any trees within sky picker have a chance to cut both lightboxes
    Quad plantRoot = Globals.getPlantRoot();
    Quad somequad = plantRoot.pickWin(sky, plantRoot, false);
    Tree shadower;
    double debit;
    double newlight = 1.0;
    double minlight = .001;
    while ((somequad != null) && (newlight > minlight)) {
      if ((somequad instanceof Tree) && (somequad != this)) {
        shadower = (Tree) somequad;
        // Can shadower affect me?
        if (shadower.z > z) {
          debit = weightedSun(skylight, sunbox);

          // now do the sky and direct sun effect of this shadowing tree
          if ( (1.0 - Woods.weightDirectSun) > 0 )
            skylight = factorSky(skylight, shadower);
          if (Woods.weightDirectSun > 0)
            factorDirectSun(sunbox, shadower);

          // some behavioral bookkeeping--grow away from debiting trees
          // and skip out of this if we can
          newlight = weightedSun(skylight, sunbox);
          debit -= newlight; // change in light
          lightWannago.addDirection(x - shadower.x, y - shadower.y, debit);
        }
      }
      // next...
      somequad = somequad.pickWin(sky, plantRoot, false);
    }

    light = newlight;
    if (light <= minlight) light = 0.0;
  }

  private double weightedSun(double skylight, Lightbox sunbox) {
    double skyweight = 1.0 - Woods.weightDirectSun;
    return skylight*skyweight  + sunbox.getLightDensity() * Woods.weightDirectSun;
  }

  public double getLight() {return light;} // for debug


  /** another subroutine of bask. Check for limiting resource, which is not
   *  a quantity of feed (ala GrassWorld) but a quality factor dimming the light,
   *  in effect, by saying we can only get 50% value from our light at this
   *  nutrient locale.
   */
  protected void findNutrientFactor() {
    // this lightlevel is per unit area of equiv area circle. Use same to
    // pick on the landscape.
    double nutrientArea = 0; // none until find some....

    Quad landRoot = Globals.getLandRoot();
    Quad somequad = landRoot.pickWin(equiv, landRoot, false);
    LimitResourceQuad limiter;
    while (somequad != null) {
      if (somequad instanceof LimitResourceQuad) {
        limiter = (LimitResourceQuad) somequad;
        if (equiv.intersectMcr(limiter, crossm) != Mcr.MCR_DISJOINT) {
          nutrientArea += limiter.getQuality(x, y) * crossm.getArea();
        }
      }
      somequad = somequad.pickWin(equiv, landRoot, false);
    }

    nutrientFactor = nutrientArea/equiv.getArea();
    // the following fix is occassioned by crowns being capable of getting
    // sufficiently tiny (Ex10-16 or so) that multiplying them goes NaN.
    // this prevents NaN from propagating, and increases the chance that the
    // un-Tree dies very very soon.
    if (Double.isNaN(nutrientFactor))
      nutrientFactor = 0;
  }

  /** eqn 3 Pacala, apply annual radial increment
   *  OR growth limit as per Constant Area Increment Law
   *  We may grow away from shade, as per wannago found during bask.
   *  This is the only "move" a tree is capable of.
   */
  public void grow() {
if (z == 0) {
  System.err.println("begin grow, height 0, this=" + this);
  Simulator.crashStop();
}
    double GLI = light * Woods.getGLI(); // global light index
    // apply eqn 3
    double stemR = 0.5 * stemDiameter;
    // losing crown volume loses productive-growth capacity
    // pi*r^2*h/pi*r^2*h
    double curDepth = z-crownBottom;
    double refDepth = z-uncroppedCrownBottom;
    double crownFactor = (radius*radius*curDepth) /
                         (uncroppedRadius * uncroppedRadius * refDepth);
    double growRateLimited = params.growRateMax * nutrientFactor * crownFactor;
    double effGrowRate = (growRateLimited * GLI)  /
                    ( (params.growRateMax/params.growRateMin) + GLI) ;
    double dStemR = stemR * effGrowRate;
if (Double.isNaN(dStemR)) {
  System.err.println("grow, dStemR NaN, this=" + this);
  Simulator.crashStop();
}


    // apply constant area increment law
    double newStemR = stemR + dStemR;
    double newRingArea = Math.PI * newStemR * newStemR;
    double oldRingArea = Math.PI * stemR * stemR;
    double maxRate = (params.growRateMax * 100) /
                  ( (params.growRateMax/params.growRateMin) + 100) ;
    double refStemR = stemR*(1.0 + maxRate);
    double refArea = Math.PI * refStemR * refStemR;
    effGrowRate = (newRingArea/refArea);
if (Double.isNaN(effGrowRate)) {
  System.err.println("grow, effGrowRate2 NaN, this=" + this);
  Simulator.crashStop();
}
    // was, before 000714
    /*
    if ((newRingArea - oldRingArea) > params.growRingMax) {
      newRingArea = oldRingArea + params.growRingMax;
      newStemR = Math.sqrt(newRingArea / Math.PI);
    }
    */
    // this ends up being a very minor curb.
    if ((newRingArea - oldRingArea) > effGrowRate*params.growRingMax) {
      newRingArea = oldRingArea + effGrowRate * params.growRingMax;
      newStemR = Math.sqrt(newRingArea / Math.PI);
    }

    getSpecies().notifyBioTrans(BiomassTransfers.GROWTH, newRingArea - oldRingArea);

if (Double.isNaN(stemDiameter)) {
  System.err.println("grow, stemDiameter NaN, this=" + this);
  Simulator.crashStop();
}
    // new lastR
    lastR = stemR;
    stemDiameter = 2.0 * newStemR;

    // now move by the amount grown (2*dr)
    dStemR = 2.0 * (newStemR - stemR);
    double[] unitv = cropWannago.getDirection();
    double[] unitv2 = lightWannago.getDirection();
    // take the average of the two wannago's.
    unitv[0] += unitv2[0]; unitv[0] /= 2;
    unitv[1] += unitv2[1]; unitv[1] /= 2;
    // the 100.0 is because x,y are in m and stemR is in cm!
    x += unitv[0] * dStemR / 100.0;
    y += unitv[1] * dStemR / 100.0;

    // update everything affected by stem diameter change
    syncWithStem();
    // System.err.println(getSpecies().getName() + Globals.pointerString(this) +
    // " grew from " + lastR + " to " + newStemR + ", tree=" + this);
  }

  /** Reproduce doesn't implement Sortie's model, though it's sort of following
   *  the spirit of it.
   */
  public void reproduce() {
    Random rands = Globals.getRandomSource();
    double distance;

    // 0. How many? Use whole part of answer as a given, fractional part as probability.

    // fecundity proportional to linear dimension cubed? with bigFertility
    // defined as fertility at maxHeight?
    double dnumKids = params.bigFertility * Math.pow((z/params.maxHeight), 3);

    // Sortie paper divides by 100 cm, comparing tree to an absolute.  They also
    // made it proportional to square instead of cube....
    // double dnumKids = params.bigFertility * stemDiameter * stemDiameter / 10000;

    int numKids = Math.round(Math.round(Math.floor(dnumKids)));
    dnumKids -= numKids;
    if (rands.nextDouble() < dnumKids)
      numKids++;
    for (int i=0; i<numKids; i++) {

      // 1. for each, how far? Since we're not using quadrats like Sortie, any
      // discrete (a la Poisson) distribution is meaningless--we want a continuous distance.
      // tests show the mean of this thing is slightly less than the mean desired.
      // like on 10,000 trials, mean 10 desired -> mean 9.97 actual. Close enuf.

      distance = (params.recruitMeanD * Math.exp(rands.nextDouble()) ) / (Math.E - 1.0);

      // 2. OK, make a kid.
      Speciesable lbaby = makeKid();
      Tree baby = (Tree) lbaby;
      double direx = 2.0 * Math.PI * rands.nextDouble();
      baby.x = x + ( distance * Math.cos(direx) );
      baby.y = y + ( distance * Math.sin(direx) );

      // do this before wrap-killing or population gets out of whack
      getSpecies().notifyBirth();

      Mcr limits = Globals.getWorldMcr();
      double sizex = limits.ux - limits.lx;
      double sizey = limits.uy - limits.ly;

      // 3. possibly stillborn, or wrapped into world
      if (baby.x > limits.ux) {
        if (Woods.isWrapX()) {
          baby.x = ((baby.x - limits.lx) % sizex) + limits.lx;
        } else {
          baby.die();
          return; // don't put baby on tree
        }
      } else if (baby.x < limits.lx) {
        if (Woods.isWrapX()) {
          baby.x = ((baby.x - limits.lx) % sizex) + sizex;
          baby.x = (baby.x % sizex) + limits.lx;
        } else {
          baby.die();
          return; // don't put baby on tree
        }
      }
      if (baby.y > limits.uy) {
        if (Woods.isWrapY()) {
          baby.y = ((baby.y - limits.ly) % sizey) + limits.ly;
        } else {
          baby.die();
          return; // don't put baby on tree
        }
      } else if (baby.y < limits.ly) {
        if (Woods.isWrapY()) {
          baby.y = ((baby.y - limits.ly) % sizey) + sizey;
          baby.y = (baby.y % sizey) + limits.ly;
        } else {
          baby.die();
          return; // don't put baby on tree
        }
      }

      // 4. Although logically, baby would die now if fell too close to
      // someone else's trunk, we leave it to the elder to kill the baby.
      // hang on quad tree
      baby.setMcr();
// System.err.println("new baby: " + baby);
      Globals.getPlantRoot().addNewbie(baby);
    }
  }

  public StageObserver getStageObserver() {
    if (getSpecies().getHoldData() == null)
      makeHeightObserver();
    return (StageObserver) getSpecies().getHoldData();
  }

  /** The activities of a Tree per step. One step = one year. */
  public void step() {
    // added to tree -this- step--skip out
    if ( (Globals.getCurrentTime() > 0) && (getAge() == 0))
      return;

    getSpecies().notifyAlive(this); // for stages

    // if < recruitAge, this tree is still growing to recruitRadius,
    // and does nothing active in the meantime. It is alive....
    if ( (Globals.getCurrentTime() >= params.recruitAge) && (getAge() < params.recruitAge)) {
      // nah, isn't included in basal area, either, since not right yet.
      // getSpecies().notifyBiomass(getBiomass());
      return;
    }

    bask();
    grow();
    if (shallIDie()) {
      die();
      return;
    }
    reproduce();
    getSpecies().notifyBiomass(getBiomass());
  }

  /** Tree uses basal area as biomass. */
  public double getBiomass() {
    double r = stemDiameter / 2.0;
    double cm2 = Math.PI * r * r;
    return cm2 / 10000.0; // return m^2 instead of cm^2
  }

  /** Try W = c d^2.5 for biomass ala Landsberg 000711. */
  /*
  public double getBiomass() {
    double cw = 5700; // units are kg?... don't cancel..."reasonable value"
    double mass = cw * Math.pow(stemDiameter, 2.5);
    return mass;
  }
  */
  /** Is R sigmoidal? Only really useful with single tree...*/
  /*
  public double getBiomass() {
    return stemDiameter/2.0;
  }
  */

  public boolean isFullGrown() {return fullGrownFlag;}

  public double getCrownBottom() {return crownBottom;}


  /** Make a gizmo for interpreting tree sizes for plots. */

  public void makeHeightObserver() {
    HeightStageObserver hso = new HeightStageObserver();
    hso.setNumStages(3);
    hso.setStageMaxHeight(params.maxHeight);
    // heightInitSlope is in m/cm
    double minHeight =
      params.maxHeight * (1.0 -
      Math.exp( - params.recruitRadius * (params.heightInitSlope/params.maxHeight) ) );
    hso.setStageMinHeight(minHeight);
    getSpecies().setHoldData(hso);
  }

  /** Paint myself into the world so I can show shadow (height, sortof) */
  public void paint(QuadDisplay qd) {
    final int minCircle = 5; // if circles < this, draw a rect
    final int minTrunk = 10; // if circle < this, skip trunk
    Graphics graphics = qd.getOffscreen();
    double squareR = McrCircle.SqrtPiOver2 * radius;
    int pc = qd.w2p(2 * squareR); // if circle < minCircle pixels, speed it up.
    synchronized (graphics) {
      if (pc < minCircle) {
        // don't waste time treating it as a circle--also draws smaller
        qd.drawRect(qd.getColor(color), x - squareR, y - squareR, x + squareR, y + squareR, true);
        return;
      }
      if (pc < minTrunk) {
        // skip the trunk nonsense
        qd.drawCircle(color, x, y, radius, false);
        return;
      }
      if (Woods.isShowShadows()) {
        shadow = getPaintShadow(); // ground level inscribed shadow of me
        graphics.setXORMode(params.xorColor);
        qd.drawOval(params.shadowColor, shadow.lx, shadow.ly, shadow.ux, shadow.uy, true);
        graphics.setPaintMode();
      }
      qd.drawCircle(color, x, y, radius, false);
      double stemR = 0.5*stemDiameter/100.0;
      pc = qd.w2p(2 * stemR * McrCircle.SqrtPiOver2); // if trunk < 4 pixels, speed it up.
      if (pc < minCircle) {
        stemR *= McrCircle.SqrtPiOver2;
        qd.drawRect(qd.getColor(color), x - stemR, y - stemR, x + stemR, y + stemR, true);
      } else
        qd.drawCircle(color, x, y, stemR, true); // trunk
      if (Woods.isShowHeights()) {
        String label = "" + TruncVal.truncVal(z,3);
        double shortR = 0.7*radius;
        qd.drawString(qd.getColor(color), label, x - shortR, y + 2*stemR, x+shortR, y+radius);
      }
    }
  }

  // =======================================================================
  // routines below this line have to do with OOP practice, mechanics of
  // getting born into the world, etc. No more logic regarding species behavior.
  // =======================================================================

  public Tree() {
    this(new TreeParams());
  }

  public Tree(TreeParams tp) {
    super();
    setParams(tp);
    biomassId = 1; // report me as basal area for biomass, resource id 1
    setType(Quad.QUAD_DRAWMYSELF); // tells quad display not to draw me.
    cropWannago = new Direction();
    lightWannago = new Direction();
  }

  protected void setParams(TreeParams tp) {params = tp;}
  protected TreeParams getParams() {return params;}

  public String toString() {
    final int sigdigits = 5;
    return getSpecies().getName() + Globals.pointerString(this) + "[" +
      "height(m)=" + TruncVal.truncVal(z, sigdigits) +
      ",stemD(cm)=" + TruncVal.truncVal(stemDiameter, sigdigits) +
      ",crownR(m)=" + TruncVal.truncVal(radius, sigdigits) +
      ",x=" + TruncVal.truncVal(x, sigdigits) +
      ",y=" + TruncVal.truncVal(y, sigdigits)
 ;
// +     ",birthday=" + birthday +
//      ",super=" + super.toString();
  }

  public void setFrom(Tree other) {
    super.setFrom(other);
    params = other.params; // redundant, but better safe than sorry....
    // defining settings handled by TreeParams params
    // here we need only worry about working settings and initialization, as needed.
    // 000627: I'm not sure why this is here instead of in makeKid, but while I'm
    // do a huge mechanical blitz, hardly the time to introduce side effects....
    lastR = params.recruitRadius;
    nutrientFactor = 1.0;

    // do this here, since it's kinda/sorta "after params set"
    // Mac's really need websafe colors only--arbitrary colors map
    // to a ghastly white
    int colorchunk = 51; // 51, 102, 153, etc. websafe colors
    if (params.shadowColor == null) {
      Color c = QuadDisplay.getColor(color);
      int r = (c.getRed() / (2*colorchunk)) * colorchunk;
      int g = (c.getGreen() / (2*colorchunk)) * colorchunk;
      int b = (c.getBlue() / (2*colorchunk)) * colorchunk;
      params.shadowColor = new Color ( r, g, b );
    }
    if (params.xorColor == null)
      params.xorColor = Color.black;

    // we're telling species to hold a StageObserver for us--
    // if none, make one now, since we've clearly got the info
    // by now to make one.
    if (getSpecies().getHoldData() == null) {
      makeHeightObserver();
    }
  }

  public Object clone() {
    return makeKid();
  }

  public Speciesable makeKid() {
    Tree kid = new Tree(params);
    kid.setFrom(this);
    kid.fullGrownFlag = false;
    kid.stemDiameter = 2 * params.recruitRadius; // their term for initRadius
    // must initialize radius & crownbottom before sync, because sync is incremental
    kid.radius = params.crownToStem * kid.stemDiameter;
    kid.syncWithStem();
    // chicken and egg problem--syncWithStem sets z first time, so must correct
    // its conclusions re the crownBottom.
    kid.crownBottom = kid.uncroppedCrownBottom;
    kid.setBirthday(Globals.getCurrentTime());
    getSpecies().notifyBirthToMom(this);
    return (Speciesable) kid;
  }

  protected static String[] props = { "recruitRadius", "maxHeight", "heightInitSlope",
    "crownToStem", "crownDepthToHeight", "maxCrownRadius", "lightCoeff", "growRateMax",
    "growRateMin", "growRingMax", "mortMax", "mortDecay", "mortRandom",
    "recruitMeanD", "bigFertility" };
  public String[] getPropertyNames() {return props;}
  public double getMaxHeight(){return params.maxHeight;}
  public void setMaxHeight(double v) {params.maxHeight = v;}
  public double getHeightInitSlope(){return params.heightInitSlope;}
  public void setHeightInitSlope(double v) {params.heightInitSlope = v;}
  public double getMaxCrownRadius(){return params.maxCrownRadius;}
  public void setMaxCrownRadius(double v) {params.maxCrownRadius = v;}

  public double getCrownDepthToHeight(){return params.crownDepthToHeight;}
  public void setCrownDepthToHeight(double v) {params.crownDepthToHeight = v;}
  public double getCrownToStem(){return params.crownToStem;}
  public void setCrownToStem(double v) {params.crownToStem = v;}
  public double getLightCoeff(){return params.lightCoeff;}
  public void setLightCoeff(double v) {params.lightCoeff = v;}
  public double getGrowRateMax(){return params.growRateMax;}
  public void setGrowRateMax(double v) {params.growRateMax = v;}
  public double getGrowRateMin(){return params.growRateMin;}
  public void setGrowRateMin(double v) {params.growRateMin = v;}
  public double getGrowRingMax(){return params.growRingMax;}
  public void setGrowRingMax(double v) {params.growRingMax = v;}
  public double getMortMax(){return params.mortMax;}
  public void setMortMax(double v) {params.mortMax = v;}
  public double getMortDecay(){return params.mortDecay;}
  public void setMortDecay(double v) {params.mortDecay = v;}
  public double getMortRandom(){return params.mortRandom;}
  public void setMortRandom(double v) {params.mortRandom = v;}
  public double getRecruitMeanD(){return params.recruitMeanD;}
  public void setRecruitMeanD(double v) {params.recruitMeanD = v;}
  public double getBigFertility(){return params.bigFertility;}
  public void setBigFertility(double v) {params.bigFertility = v;}
  public double getRecruitRadius(){return params.recruitRadius;}
  public void setRecruitRadius(double v) {
    params.recruitRadius = v;
  }
  public int getRecruitAge(){return params.recruitAge;}
  public void setRecruitAge(int v) {params.recruitAge = v;}
}

