//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 2000, All rights reserved
//Author:       Ginger Booth, ginger@gingerbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * vfb, 001214, original, Ofer's sit&wait spider
 */

package gecko.zoo;
import gecko.*;

public class SittingSpider extends ScaredyGrazerCarnie implements HasSpeciesInit {
  // want non-overlapping attack radii, give up after 10 tries and stay here.
  private final static int maxMoveTries = 10;

  // defining params
  // attack radius uses smell radius--can smell 'em, can eat 'em
  protected double moveProbability;
  protected double refugeHuntSuccess;
  protected double normalHuntSuccess;

  private static McrCircle searchWin = new McrCircle();

  //menu machinery
  private static String[] myprops = { "moveProbability",
    "refugeHuntSuccess", "normalHuntSuccess" };
  protected static String[] props;
  static {
    String[] firsthalf = ScaredyGrazerCarnie.props;
    props = new String[firsthalf.length + myprops.length];
    for (int i=0; i<myprops.length; i++) {
      props[i] = myprops[i];
    }
    for (int i=0; i<firsthalf.length; i++) {
      props[myprops.length+i] = firsthalf[i];
    }
  }
  public String[] getPropertyNames() { return props;}


  public SittingSpider() {
    super();
  }

  public Speciesable makeKid() {
    SittingSpider kid = new SittingSpider();
    kid.setFrom(this);
    getSpecies().notifyBirthToMom(this);
    return kid;
  }
  public Object clone() {
    return makeKid();
  }

  public void setFrom(SittingSpider other) {

    super.setFrom(other);
    this.moveProbability = other.moveProbability;
    this.normalHuntSuccess = other.normalHuntSuccess;
    this.refugeHuntSuccess = other.refugeHuntSuccess;
  }


  public boolean attacks(Critter other) {

    boolean answer;

    if (other instanceof Exposureable) {

      if ( ((Exposureable)other).exposure().isNullMcr() ) {

        answer = Globals.getRandomDouble() < refugeHuntSuccess;

        return answer;

      } else {

        answer = super.attacks(other) && (Globals.getRandomDouble() < normalHuntSuccess);

        return answer;

      }

    }

    return false; // we only hunt Exposurable (ScaredyGrazer && OferStateGrazer)

  }


  /** Hunt greatly simplified for SittingSpider.

   */
  public void hunt() {
    // Can eat a gopper?
    Critter gopper = findCritter(radius * getSmellRadius(), true, Globals.getMobileRoot());
    if (gopper != null) {
      doEat(gopper);
    }

  }


  /** Are picking and choosing things super does, my rules... Don't leave the

   *  landRoot, you'll fall asleep and starve out there... Move at random

   *  intervals, never overlap other SittingSpider's attackRadius, ignore any

   *  direction choices of super's...

   */

  public void move() {

    // moving day?

    if (Globals.getRandomDouble() >= this.moveProbability)

      return;


    // ok, try a move

    Mcr landMcr = (Mcr) Globals.getLandRoot(); // root=quad=mcr

    double wasx = getX();

    double wasy = getY();

    int tries = 0;


    // we exit from inside here if find a solution

    do {

      // random direction, move by...attack radius, I guess.

      double direction = Math.PI * 2.0 * Globals.getRandomDouble();

      double distance = radius * getAttackRadius();

      x = wasx + (Math.cos(direction) * distance);

      y = wasy + (Math.sin(direction) * distance);

      // inside landscape?

      if (landMcr.intersectMcr(this, null) != MCR_CONTAINS) {

        // no good--snap into landscape

        this.snapInto(landMcr);

        setXY(this.getCenterX(), this.getCenterY());

      }

      if (!spidersOverlap()) {

        return;

      }


      tries++;

    } while (tries < maxMoveTries);


    Globals.log("Warning, spider was unable to move.");

    x = wasx;

    y = wasy;

  }


  /** Called only on individuals created on species birthday. In this case,

   *  make sure hunting ranges don't overlap. */

  public void speciesInit() {

    final boolean noPickInvis = false;

    int tries = 0;

    SittingSpider ss;


    // the birth area for this species, in case need to relocate

    Mcr area = getSpecies().getInitArea();

    double rangex = area.ux - area.lx;
    double rangey = area.uy - area.ly;


    Quad huntroot = Globals.getMobileRoot();

    // actually want to pick my-attack vs his-attack, but...

    searchWin.circleToSquare(x, y, radius * 2 * getSmellRadius());

    Quad other = huntroot.pickWin(searchWin, huntroot, noPickInvis);

    while (spidersOverlap()) {

      // relocate me

      x = area.lx + Globals.getRandomDouble() * rangex;

      y = area.ly + Globals.getRandomDouble() * rangey;

      tries++;

      if (tries > maxMoveTries) {

        Globals.log("WARNING: some spiders overlap at beginning.");

        return;

      }

    }

  }


  /** true if I overlap any other SittingSpider by attackRadii */

  protected boolean spidersOverlap() {

    final boolean noPickInvis = false;

    SittingSpider ss;


    Quad huntroot = Globals.getMobileRoot();

    // actually want to pick my-attack vs his-attack, but...

    searchWin.circleToSquare(x, y, radius * 2 * getSmellRadius());

    Quad other = huntroot.pickWin(searchWin, huntroot, noPickInvis);

    while (other != null) {

      if (other != this) {

        if (other instanceof SittingSpider) {

          // candidate, does my attack radius overlap another spider's?

          ss = (SittingSpider) other;

          if ( McrCircle.crossCircles(x, y, radius * getSmellRadius(),

                                      ss.x, ss.y, ss.radius * ss.getSmellRadius())

               != 0.0 )

          {

            return true;

          }

        }

      }

      // next

      other = other.pickWin(searchWin, huntroot, noPickInvis);

    }

    return false;

  }


  public void setAttackRadius(double d) {setSmellRadius(d);}
  public double getAttackRadius() {return getSmellRadius();}
  public double getMoveProbability(){return moveProbability;}
  public void setMoveProbability(double v) {moveProbability = v;}
  public double getRefugeHuntSuccess(){return refugeHuntSuccess;}
  public void setRefugeHuntSuccess(double v) {refugeHuntSuccess = v;}
  public double getNormalHuntSuccess(){return normalHuntSuccess;}
  public void setNormalHuntSuccess(double v) {normalHuntSuccess = v;}


}
