
//Title:        Indeco Steel Simulator
//Version:      
//Copyright:    Copyright (c) 1999
//Author:       Joseph Guse & Ginger Booth
//Company:      Yale FES/CCE
//Description:  Individual-based economic simulator
//of the steel industry, ore->consumer,
//with transportation.

/** A MetaPlot is a collection of SumTraces all set to be deployed to
 *  whatever StatDumper (Plotter, file writer, whatever) when asked for.
 *  In other words, it's a plot specification, minus the Plotter and/or file
 *  dumper.
 *
 *  Both UI and StatMinder operate on StatDumpers via MetaPlot. Lifecycle:
 *  1. StatMinder creates the MetaPlot, adds traces, etc, as needed.
 *  2. MetaPlot.show calls UI to create its dumpers.
 *  3. MetaPlot.step steps all the traces.
 *  4. MetaPlot.clear and finalize both finalize the traces.
 *
 *  Changing a trace along the way is the trace's job, transparent to MetaPlot.
 *  MetaPlot.addTrace is called from StatMinder.
 *
 *  Note that MetaPlot steps traces regardless of whether any dumpers have been
 *  specified. Thus, if want stats available to show on demand, create the
 *  MetaPlot with no dumpers, so it has the historical data available.
 */

/** Adopted from Indeco for Gecko-in-CourseWare, 990708, vfb. */

package stats;


import java.util.*;
import java.awt.*;

public class MetaPlot {
  private String title;
  private Vector traces = new Vector(10,1); // all PlotTrace
  private boolean mayTruncateTraces = false; // not yet turned on....
  private UI ui; // need to know how to contact the user interface
  private boolean needLegend = false; // if plot needs legend

  public MetaPlot(String title, UI ui) {
    this.title = title;
    mayTruncateTraces=false;
    this.ui = ui;
  }

  public boolean getNeedLegend() { return needLegend;}
  public void setNeedLegend(boolean flag) { needLegend = flag;}

  protected UI getUI() { return ui;}
  public Vector getTraces() { return traces;}

  public void setMayTruncateTraces(boolean b){mayTruncateTraces = b;}
  public boolean getMayTruncateTraces(){return mayTruncateTraces;}

  /** doesn't need to do anything at present...just being consistent.*/
  public void init() {}

  /** to reset for next run, get rid of all traces. Calls finalize.
   */
  public void clear() {
    finalize();
    traces = new Vector(10,1);
  }

  public void reset() {
    for (int trace=0; trace < traces.size(); trace++) {
      ((SumTrace) traces.elementAt(trace)).clear();
    }
  }


  public void finalize() {
    // atomize vectors so they garbage-collect promptly
    for (int trace=0; trace < traces.size(); trace++) {
      ((SumTrace) traces.elementAt(trace)).clear();
    }
    traces.removeAllElements();
    traces = null;
  }

  public String getName() { return title; }
  public void setName(String name) { title = name; }

  /** Name for the file is title with whitespace squeezed out. */
  public String getNameForFile() {
    StringBuffer buffy = new StringBuffer(256);
    for (int i = 0; i < title.length(); i++) {
      if (title.charAt(i) != ' ') {
        buffy.append(title.charAt(i));
      }
    }
    return buffy.toString();
  }

  public void addTrace(SumTrace newTrace) {
    traces.addElement(newTrace);
    Vector outputs = ui.getStatDumpers(this);
    if (outputs == null)
      return;
      
    StatDumper sink;
    for (int i=0; i<outputs.size(); i++) {
      sink = (StatDumper) outputs.elementAt(i);
      sink.addTrace(newTrace);
    }
  }

  public boolean hasTrace(String name) {
    for (int i=0;i<traces.size(); i++) {
      if ( ((SumTrace)traces.elementAt(i)).getLabel().equals(name) )
        return true;
    }
    return false;
  }

  public SumTrace getTrace(String name) {
    for (int i=0;i<traces.size(); i++) {
      if ( ((SumTrace)traces.elementAt(i)).getLabel().equals(name) )
        return ((SumTrace)traces.elementAt(i));
    }
    return null;
  }


  public void show() {
    ui.makeStatDumpers(this);
  }
  
  public void showOn(StatDumper statSink) {
    for (int i=0; i< traces.size(); i++)
      statSink.addTrace((SumTrace) traces.elementAt(i));
    statSink.update();
  }

  /** Calls step on all member SumTraces, some of which may do something. */
  public void step() {
    for (int trace=0; trace < traces.size(); trace++) {
      ((Steppable) traces.elementAt(trace)).step();
    }
  }
}
