
//Title:        Indeco Steel Simulator
//Version:      
//Copyright:    Copyright (c) 1999
//Author:       Joseph Guse & Ginger Booth
//Company:      Yale FES/CCE
//Description:  Individual-based economic simulator
//of the steel industry, ore->consumer,
//with transportation.

/** StatBrokerables are basically a stat-getters for SumTrace, etc., statistics
 *  routines. Although methods could be used here, they're highly inefficient.
 *  Better to provide simple broker classes for assorted stats to get them.
 *
 *  Normalize and weight:
 *  Typically, statistics involve:
 *    simple sums
 *    averages
 *  And if you put them on the same plot, they must be of comparable orders of
 *  magnitude. So if 1 unit steel production takes 100 units iron ore input, 10
 *  units human effort, 1.3 units energy, a sensible plot would use the steel
 *  output normalized at 1.0, iron ore at 1/100, humans at 1/10, energy at 1/1.3,
 *  and call the units "steel equivalents" or something. Otherwise, the steel and
 *  energy components of the plot would appear as straight lines.
 *
 *  Weighting has to do with weighted averages. Easiest example is price. The
 *  average retail price is the sum over all retailers of volume * price,
 *  divided by the sum of volume. In this case, volume is the weight.
 *
 *  Weights are returned raw. They are not multipled by the normalizer.
 *
 *  Values are returned normalized.
 *
 *  The normalizer should ALWAYS be initialized to 1.0 until told otherwise.
 *
 *  Return INVALID_STAT for all invalid values.
 */

/** Adopted from Indeco for Gecko-in-CourseWare, 990708, vfb. */

package stats;


public interface StatBrokerable {
  public static final double INVALID_STAT = Double.NaN;

  /** Who does this broker get stats from? */
  public Object getStatSource();
  /** getValue() always returns normalized, unweighted value. */
  public double getValue();
  /** getWeight()'s values are never normalized. */
  public double getWeight();

  /** normalizer should always default to 1.0 until told otherwise.
   *  Unlike weight and value, which may well vary from simulation
   *  step to step, a normalizer is a constant.  Hence, it's the only
   *  field for which a "set" is really possible. (The last thing we
   *  want is the Agent changing in midstream...)
   *
   *  Normalizer is a multiplier applied when getValue (=value*normalizer)
   *  normalizer is not applied to the weight. 
   */
  public double getNormalizer();
  public void setStatSource(Object o);
  public void setNormalizer(double d);
}
